#pragma once

/* Genode includes */
#include <session/session.h>
#include <base/rpc.h>
#include <dataspace/capability.h>

namespace Tcb_info
{
	using namespace Genode;
	using Genode::uint64_t;

	using Security_domain = Genode::String<128>;

	enum {
		MERKLE_TREE_SIZE  = 3 * 1024 * 1024, // maximal size in bytes
	};

	enum class Tcb_type {
		Platform        = 1,
		Security_domain = 2,
		Tee             = 4,
	};

	struct Session;

	typedef uint64_t Transaction_handle;

	class Buffer_too_small      : public Exception { };
	class Transaction_failed    : public Exception { };
	class Transaction_not_found : public Exception { };
	class Wrong_hash_size       : public Exception { };
}


struct Tcb_info::Session : Genode::Session
{
	using Ram_limit_exceeded = Genode::Quota_guard<Genode::Ram_quota>::Limit_exceeded;
	using Cap_limit_exceeded = Genode::Quota_guard<Genode::Cap_quota>::Limit_exceeded;

	/**
	 * \noapi
	 */
	static const char *service_name() { return "Tcb_info"; }

	/*
	 * A tcb_info session consumes a dataspace capability for the server's
	 * session-object allocation, its session capability, and a dataspace
	 * capability for the communication buffer.
	 */
	enum { CAP_QUOTA = 7 };

	/*
	 * start a transaction to get a platform state.
	 *
	 * @param   type               type of the platform state to obtain
	 * @param   name_buffer        name for types that need an address
	 * @param   name_size          size of the name
	 *
	 * @throws  Buffer_too_small   the transaction buffer is to small
	 */
	virtual Transaction_handle begin_transaction(Tcb_type type) = 0;

	/**
	  * Get the merkle tree of the platform state.
	  */
	virtual size_t get_tcb_info_tree(Transaction_handle transaction_handle, uint8_t* tree_buffer, size_t tree_buffer_size) = 0;

	virtual void provide_additional_hash(const char* hash_buffer, size_t hash_size,
	                                     const char* hash_name, size_t name_size) = 0;

	virtual size_t get_state() = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_begin_transaction, Transaction_handle, _begin_transaction,
	                 GENODE_TYPE_LIST(Ram_limit_exceeded, Cap_limit_exceeded),
	                 Tcb_type);
	GENODE_RPC_THROW(Rpc_get_tcb_info_tree, size_t, _get_tcb_info_tree,
	                 GENODE_TYPE_LIST(Buffer_too_small, Transaction_not_found, Transaction_failed,
	                                  Ram_limit_exceeded, Cap_limit_exceeded),
	                 Transaction_handle);
	GENODE_RPC_THROW(Rpc_provide_additional_hash, void, _provide_additional_hash,
	                 GENODE_TYPE_LIST(Buffer_too_small, Wrong_hash_size,
	                                  Ram_limit_exceeded, Cap_limit_exceeded),
	                 size_t, size_t);
	GENODE_RPC(Rpc_dataspace, Dataspace_capability, _dataspace);
	GENODE_RPC(Rpc_get_state, size_t, _get_state);


	GENODE_RPC_INTERFACE(Rpc_begin_transaction,
	                     Rpc_get_tcb_info_tree,
	                     Rpc_provide_additional_hash,
	                     Rpc_dataspace,
	                     Rpc_get_state);
};

